@@ -43,12 +43,18 @@ pub(crate) struct AscentIr {
    pub relations_full_indices: HashMap<RelationIdentity, IrRelation>,
    pub lattices_full_indices: HashMap<RelationIdentity, IrRelation>,
    // pub relations_no_indices: HashMap<RelationIdentity, IrRelation>,
-   pub relations_initializations: HashMap<RelationIdentity, Rc<Expr>>,
+   pub relations_metadata: HashMap<RelationIdentity, RelationMetadata>,
    pub rules: Vec<IrRule>,
    pub declaration: Declaration,
    pub config: AscentConfig,
 }
 
+#[derive(Clone, Default)]
+pub(crate) struct RelationMetadata{
+   pub inititialization: Option<Rc<Expr>>,
+   pub attributes: Rc<Vec<Attribute>>,
+}
+
 pub(crate) struct IrRule {
    pub head_clauses: Vec<IrHeadClause>,
    pub body_items: Vec<IrBodyItem>,
@@ -133,6 +139,7 @@ pub(crate) fn compile_ascent_program_to_hir(prog: &AscentProgram) -> syn::Result
    let mut relations_ir_relations: HashMap<RelationIdentity, HashSet<IrRelation>> = HashMap::new();
    let mut relations_full_indices = HashMap::new();
    let mut relations_initializations = HashMap::new();
+   let mut relations_metadata = HashMap::new();
    // let mut relations_no_indices = HashMap::new();
    let mut lattices_full_indices = HashMap::new();
    for rel in prog.relations.iter(){
@@ -165,6 +172,13 @@ pub(crate) fn compile_ascent_program_to_hir(prog: &AscentProgram) -> syn::Result
       if let Some(init_expr) = &rel.initialization {
          relations_initializations.insert(rel_identity.clone(), Rc::new(init_expr.clone()));
       }
+      relations_metadata.insert(
+         rel_identity.clone(),
+         RelationMetadata {
+            inititialization: rel.initialization.clone().map(|i| Rc::new(i)),
+            attributes: Rc::new(rel.attrs.clone())
+         }
+      );
       // relations_no_indices.insert(rel_identity, rel_no_index);
    }
    for (ir_rule, extra_relations) in ir_rules.iter(){
@@ -189,7 +203,7 @@ pub(crate) fn compile_ascent_program_to_hir(prog: &AscentProgram) -> syn::Result
       relations_ir_relations,
       relations_full_indices,
       lattices_full_indices,
-      relations_initializations,
+      relations_metadata: relations_metadata,
       // relations_no_indices,
       declaration,
       config: AscentConfig::new(prog.attributes.clone())?
@@ -202,7 +216,12 @@ fn compile_rule_to_ir_rule(rule: &RuleNode, prog: &AscentProgram) -> syn::Result
    fn extend_grounded_vars(grounded_vars: &mut Vec<Ident>, new_vars: impl IntoIterator<Item = Ident>) -> syn::Result<()> {
       for v in new_vars.into_iter() {
          if grounded_vars.contains(&v) {
-            return Err(Error::new(v.span(), format!("variable {} shadows a variable with the same name", v)));
+            // TODO may someday this will work
+            // let other_var = grounded_vars.iter().find(|&x| x == &v).unwrap();
+            // let other_err = Error::new(other_var.span(), "");
+            let err = Error::new(v.span(), format!("'{}' shadows another variable with the same name", v));
+            // err.combine(other_err);
+            return Err(err);
          }
          grounded_vars.push(v);
       }
@@ -219,7 +238,12 @@ fn compile_rule_to_ir_rule(rule: &RuleNode, prog: &AscentProgram) -> syn::Result
             }
 
             if bitem_ind == 1 && first_two_items_simple_clauses{
-               let mut self_vars = bcl.args.iter().filter_map(|arg| expr_to_ident(arg.unwrap_expr_ref())).collect::<HashSet<_>>();
+               let mut self_vars = HashSet::new();
+               for var in bcl.args.iter().filter_map(|arg| expr_to_ident(arg.unwrap_expr_ref())) {
+                  if !self_vars.insert(var) {
+                     first_two_items_simple_clauses = false;
+                  }
+               }
                for cond_cl in bcl.cond_clauses.iter(){
                   let cond_expr = cond_cl.expr();
                   let expr_idents = expr_get_vars(&cond_expr);
